/**********************************************************************
*   Copyright: (C)2021 LingYun IoT System Studio <www.weike-iot.com>
*      Author: GuoWenxue<guowenxue@gmail.com> QQ: 281143292
* Description: BC28 NB-IoT module AT command source code
*
*
*   ChangeLog:
*        Version    Date       Author            Description
*        V1.0.0  2022.05.1  GuoWenxue      Release initial version
***********************************************************************/

#ifndef INC_BC28_H_
#define INC_BC28_H_

#include "usart.h"
#include "atcmd.h"


/* Check NB-IoT module name */
extern int atcmd_check_module_name(char *modname, int size);


/* Check NB-IoT module software version */
extern int atcmd_check_module_version(void);


/* Check module IMEI(International Mobile station Equipment Identity) */
extern int atcmd_check_module_imei(char *imei, int size);


/* Check SIM card ready or not by check IMSI(International Mobile Subscriber Identity) */
extern int atcmd_check_sim_ready(void);

/* Send AT command to reset NB-IoT module */
extern int atcmd_reset_module(void);


/* Send AT command to set the UE Functionality. */
extern int atcmd_set_cfun(int fun);

/* Send AT command to get the UE Functionality. */
extern int atcmd_get_cfun(void);


/* Send AT command to set supported bands. */
extern int atcmd_set_bands(char *bands);

/* Send AT command to get supported bands. */
extern int atcmd_get_bands(void);

/* Send AT command to set auto connect or not. */
extern int atcmd_set_autoconnect(int enable);

/* Send AT command to set NB-IoT network attach. */
extern int atcmd_set_attach(int attach);

/* Send AT command to get NB-IoT network attach status. */
extern int atcmd_get_attach(void);


/* Send AT command to check NB-IoT network signal. */
extern int atcmd_check_sginal(void);

/* Send AT command to check NB-IoT register status. */
extern int atcmd_check_register(void);

/* Send AT command to check NB-IoT network IP address. */
extern int atcmd_check_ipaddr(char *ipaddr, int size);

/* Send AT command to check NB-IoT operator MCC number. */
extern int atcmd_check_operator(char *mcc, int size);


/*+--------------------------------------------------------+
 *|   NB-IoT module register to IoT platform functions     |
 *+--------------------------------------------------------+
 */

/* Send AT command to set register IoT platform server. */
extern int atcmd_set_iot_platform(char *server);

/* Send AT command to check register IoT platform server. */
extern int atcmd_check_iot_platform(char *server);

/* Send AT command to set data encrypto mode */
enum
{
	MODE_ENCRYPT_NONE,
	MODE_ENCRYPT_DTLS_STD,
	MODE_ENCRYPT_DTLS_PLUS,
};

enum
{
	NAT_ENABLE,
	NAT_DISABLE,
};

extern int atcmd_set_iot_encrypt_mode(int mode, int nat);


/* Send AT command to trigger IoT platform register */
enum
{
	TRIGGER_REGISTER,
	TRIGGER_DEREGISTER,
};
extern int atcmd_trigger_iot_register(int mode);

/* Send AT command to set IoT platform register mode */
enum
{
	MODE_REG_MANUAL,
	MODE_REG_AUTO,
	MODE_REG_DISABLE,
};
extern int atcmd_set_iot_register_mode(int mode);


/* Send AT command to query IoT platform register status */
extern int atcmd_get_iot_register_status(void);


/* Send AT command to send data to IoT platform */
extern int atcmd_iot_send_data(char *data);

#endif /* INC_BC28_H_ */
