/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * @file           : main.c
  * @brief          : Main program body
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; Copyright (c) 2021 STMicroelectronics.
  * All rights reserved.</center></h2>
  *
  * This software component is licensed by ST under BSD 3-Clause license,
  * the "License"; You may not use this file except in compliance with the
  * License. You may obtain a copy of the License at:
  *                        opensource.org/licenses/BSD-3-Clause
  *
  ******************************************************************************
  */
/* USER CODE END Header */
/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "i2c.h"
#include "tim.h"
#include "usart.h"
#include "gpio.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */
#include <string.h>
#include "sht30.h"
#include "oled.h"
#include "nbiot.h"
/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/
/* USER CODE BEGIN PTD */

/* USER CODE END PTD */

/* Private define ------------------------------------------------------------*/
/* USER CODE BEGIN PD */
/* USER CODE END PD */

/* Private macro -------------------------------------------------------------*/
/* USER CODE BEGIN PM */

/* USER CODE END PM */

/* Private variables ---------------------------------------------------------*/

/* USER CODE BEGIN PV */
  int                 posX=2, posY=2;
/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
/* USER CODE BEGIN PFP */

/* USER CODE END PFP */

/* Private user code ---------------------------------------------------------*/
/* USER CODE BEGIN 0 */
static int report_tempRH(void);
/* USER CODE END 0 */

/**
  * @brief  The application entry point.
  * @retval int
  */
int main(void)
{
  /* USER CODE BEGIN 1 */
  uint32_t            last_time = 0;
  int                 i, rv;
  char                lcdmsg[128];
  /* USER CODE END 1 */

  /* MCU Configuration--------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* USER CODE BEGIN Init */

  /* USER CODE END Init */

  /* Configure the system clock */
  SystemClock_Config();

  /* USER CODE BEGIN SysInit */

  /* USER CODE END SysInit */

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  MX_TIM6_Init();
  MX_TIM2_Init();
  MX_USART1_UART_Init();
  MX_I2C2_Init();
  MX_USART3_UART_Init();
  /* USER CODE BEGIN 2 */

  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */

  sysled_hearbeat();
  beep_start(2, 300);
  printf("Start BearKE1 5G NB-IoT Board Example Program v1.0\r\n");

  OLED_Init();
  OLED_ShowBanner(TIME_1S*2);


  /* Check NB-IoT module exist or not */
  rv = nbiot_check_module();
  if( 0 != rv )
  {
     printf("FATAL: NB-IoT module detect failed, system pause here.\r\n");
     do {} while(1);
  }

  /* Check NB-IoT module register okay or not */
  for(i=0; i<6; i++)
  {
	  nbiot_check_status();
	  if( NBIOT_STATUS_READY != g_nbiot_status )
	  {
		  HAL_Delay(1000);
	  }
  }

  /* If module not register, initial and configure for the module */
  if( NBIOT_STATUS_READY != g_nbiot_status )
  {
	  rv = nbiot_module_init();
	  if( rv != 0 )
	  {
		  printf("ERROR: NB-IoT module initial       [Failed, rv=%d]\r\n", rv);
	  }
	  printf("ERROR: NB-IoT module initial       [Okay]\r\n");
  }


  while (1)
  {
	  if( g_nbiot_status != NBIOT_STATUS_READY )
	  {
		  nbiot_check_status();
	  }

	  if( g_nbiot_status != NBIOT_STATUS_READY )
	  {
		  printf("ERROR: Check NB-IoT module status  [Failed, status=%d]\r\n", rv);
		  HAL_Delay(200);
		  continue;
	  }

	  if( time_after(HAL_GetTick(), last_time+30000) )
	  {
		  report_tempRH();
		  last_time = HAL_GetTick(); /*update last report time */
	  }

    /* USER CODE END WHILE */

    /* USER CODE BEGIN 3 */
	  nbiot_update_status();
	  snprintf(lcdmsg, sizeof(lcdmsg), "Stat[%d] Sig[%d]", g_nbiot_status, g_nbiot_signal);
	  OLED_ShowString(posX, posY, lcdmsg, OLED_FONT16);
	  HAL_Delay(1000);
  }
  /* USER CODE END 3 */
}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{
  RCC_OscInitTypeDef RCC_OscInitStruct = {0};
  RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};

  /** Configure the main internal regulator output voltage
  */
  if (HAL_PWREx_ControlVoltageScaling(PWR_REGULATOR_VOLTAGE_SCALE1) != HAL_OK)
  {
    Error_Handler();
  }

  /** Initializes the RCC Oscillators according to the specified parameters
  * in the RCC_OscInitTypeDef structure.
  */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSE;
  RCC_OscInitStruct.HSEState = RCC_HSE_ON;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;
  RCC_OscInitStruct.PLL.PLLM = 1;
  RCC_OscInitStruct.PLL.PLLN = 20;
  RCC_OscInitStruct.PLL.PLLP = RCC_PLLP_DIV7;
  RCC_OscInitStruct.PLL.PLLQ = RCC_PLLQ_DIV2;
  RCC_OscInitStruct.PLL.PLLR = RCC_PLLR_DIV2;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }

  /** Initializes the CPU, AHB and APB buses clocks
  */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV1;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_4) != HAL_OK)
  {
    Error_Handler();
  }
}

/* USER CODE BEGIN 4 */

int report_tempRH(void)
{
	char          buf[128];
    uint32_t      temperature, humidity;
    int           rv = 0;
    int           type_report = 0x02;
    int           temp_servid = 0x000A;
    int           humd_servid = 0x0009;
    int           payload_len = 0x0004;

    if ( SHT30_SampleData(&temperature, &humidity) < 0 )
    {
    	printf("ERROR: SHT30 Sample data failure\n");
    	return -1;
    }

    memset(buf, 0, sizeof(buf));
    snprintf(buf, sizeof(buf), "%ld.%ldoC, %ld.%ld%%", (temperature>>16)&0xFFFF,
    		temperature&0xFFFF, (humidity>>16)&0xFFFF, humidity&0xFFFF);

    printf("INFO: Sample T&H value             [%s]\r\n", buf);
    OLED_ShowString(posX, posY, buf, OLED_FONT16);

    memset(buf, 0, sizeof(buf));
    snprintf(buf, sizeof(buf), "%02X%04X%04X%08lX", type_report, temp_servid, payload_len, temperature);
    if( (rv=atcmd_iot_send_data(buf)) != 0 )
    {
    	printf("ERROR: Report T&H to IoT Platform  [Failed, rv=%d]\r\n", rv);
    	rv = -2;
    	goto OUT;
    }

    HAL_Delay(1000);

    memset(buf, 0, sizeof(buf));
    snprintf(buf, sizeof(buf), "%02X%04X%04X%08lX", type_report, humd_servid, payload_len, humidity);
    if( (rv=atcmd_iot_send_data(buf)) != 0 )
    {
    	printf("ERROR: Report T&H to IoT Platform  [Failed, rv=%d]\r\n", rv);
    	rv = -3;
    	goto OUT;
    }

    printf("INFO: Report T&H to IoT Platform   [Okay]\r\n");

OUT:
	HAL_Delay(2000);
    return rv;
}

/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */
  __disable_irq();
  while (1)
  {
  }
  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */
